<?php
/*
Plugin Name: GravatarLocalCache
Website link: http://blog.splash.de/
Author URI: http://blog.splash.de/
Plugin URI: http://blog.splash.de/2010/01/04/gravatarlocalcache
Description: local cache for gravatar images.
Author: Oliver Schaal
Version: 0.9.1

    This is a WordPress plugin (http://wordpress.org) and widget
    (http://automattic.com/code/widgets/).
*/


if (!function_exists('is_admin')) {
    header('Status: 403 Forbidden');
    header('HTTP/1.1 403 Forbidden');
    exit();
}

require_once(ABSPATH . 'wp-admin/includes/file.php');

define('GLCPATH', WP_PLUGIN_DIR.'/'.dirname(plugin_basename( __FILE__ )));
define('GLCURL', WP_PLUGIN_URL.'/'.dirname(plugin_basename( __FILE__ )));

define('CACHEPATH', GLCPATH.'/cache/');
define('CACHEURL', GLCURL.'/cache/');

if (!class_exists("GravatarLocalCache")) {
    class GravatarLocalCache {

        // vars
        private static $defaultAvatar = 'mystery';

        // put all options in
        private static $options = array();

        /* __construct */
        public function __construct() {
            // get other options/vars
            // self::$defaultAvatar = get_option('avatar_default');

            //load language
            if (function_exists('load_plugin_textdomain'))
                load_plugin_textdomain('glc', GLCPATH.'/langs/', '/gravatarlocalcache/langs/');

            // get options
            self::$options = get_option('glcOptions');
            (!is_array(self::$options) && !empty(self::$options)) ? self::$options = unserialize(self::$options) : self::$options = false;

            // install default options
            register_activation_hook(__FILE__, array(&$this, 'activatePlugin'));

            // plugin administration stuff
            register_deactivation_hook(__FILE__, array(&$this, 'deactivatePlugin'));

            // admin menu
            add_action('admin_menu', array(&$this, 'showAdminMenuLink'));

            // plugin stuff
            if ( is_dir(CACHEPATH) || ( umask(0022) && @mkdir( CACHEPATH , 0755, true ) ) ) {
                add_filter('get_avatar', array(&$this, 'getGravatarLocalCacheFilter'), 1, 2);
            }
        }
        /* __construct */

        /* activatePlugin */
        public function activatePlugin() {
            if (empty($this->options)) {
                add_option('glcOptions', serialize(array(
                        'cache_time' => '72',          // 3 days...
                )));
            } else {
                // update options after plugin update
                $this->update();
            }
        }
        /* activatePlugin */

        /* deactivatePlugin */
        public function deactivatePlugin() {
            delete_option('glcOptions');
        }
        /* deactivatePlugin */

        private function update() {
            // add option, if they does not exist
            if(empty(self::$options['cache_time']))
                self::$options['cache_time'] = '72';        // 3 days...

            // update options
            update_option('glcOptions', serialize($this->options));
        }

        /* callbackGravatarLocalCache */
        private static function callbackGravatarLocalCache($matches) {
            // print '<!-- GravatarCallback: ';
            // print_r($matches);
            // print '-->';

            $_imageurl = $matches[1];

            preg_match('|http://www.gravatar.com/avatar/([^\?]+)\?s=([0-9]+)&|i', $_imageurl, $_img);

            $_name = $_img[1];
            $_file = CACHEPATH.$_name.'-'.$_img[2].'.jpg';
            $_url = CACHEURL.$_name.'-'.$_img[2].'.jpg';

            // print '<!-- GravatarHASH: ';
            // print $_hash[1];
            // print '-->';

            if (file_exists($_file) && ((time() - filemtime($_file))/60/60) < self::$options['cache_time']) {
                $_retVal = $_url;
            } else {

                if (!empty($_imageurl) ) {
                    // Download file to temp location
                    $_tmp = download_url(str_replace(utf8_encode('&amp;'), '&', $_imageurl));
                    // Set variables for storage

                    // If error storing temporarily, unlink
                    if ( is_wp_error($_tmp) ) {
                        @unlink($_tmp);

                        // lets try to fetch the default avatar
                        preg_match('/&amp;d=(.*)/i',$_imageurl,$_defaultavatar);
                        $_tmp = download_url(urldecode($_defaultavatar[1]));
                        if ( is_wp_error($_tmp) ) {
                            @unlink($_tmp);
                            return $matches[0];
                        }
                    }

                    // do the validation and storage stuff
                    if ( false === @rename( $_tmp, $_file ) ) {
                        return $matches[0];
                    } else {
                        $_stat = stat( dirname( $_file ));
                        $_perms = $_stat['mode'] & 0000666;
                        @chmod( $_file, $_perms );

                        $_retVal = $_url;
                    }
                    @unlink($_tmp);

                } else {
                    return $matches[0];
                }
            }

            return str_replace($_imageurl,$_retVal,$matches[0]);
        }
        /* callbackGravatarLocalCache */

        /* getGravatarLocalCacheFilter */
        public function getGravatarLocalCacheFilter($content, $arg) {
            return preg_replace_callback('|<img.*src=\'([^\']+)\'.*/>|i', array( &$this, 'callbackGravatarLocalCache'), $content);
        }
        /* getGravatarLocalCacheFilter */

        /* showAdminMenuLink */
        public function showAdminMenuLink()
        {
            $hook = add_options_page('GravatarLocalCache',
                (version_compare($GLOBALS['wp_version'], '2.6.999', '>') ? '<img src="' . GLCURL.'/icon.jpg' . '" width="10" height="10" alt="GravatarLocalCache - Icon" /> ' : '') . 'GravatarLocalCache',
                9,
                plugin_basename(__FILE__),
                array(&$this,
                    'showAdminMenu'
                )
            );
            if (function_exists('add_contextual_help') === true) {
                add_contextual_help($hook,
                    sprintf('<a href="http://trac.splash.de/glc">%s</a><a href="http://board.splash.de/forumdisplay.php?f=105">%s</a>',
                        __('Ticketsystem/Wiki', 'glc'),
                        __('Support-Forum', 'glc')
                    )
                );
            }
        }
        /* showAdminMenuLink */

        /* showAdminMenu */
        public function showAdminMenu()
        {

            if (!empty($_POST)) {

                if(!empty($_POST['cache_time'])) {
                    self::$options['cache_time'] = $_POST['cache_time'];
                }
                // update options
                update_option('glcOptions', serialize(self::$options));

                // echo successfull update
                echo '<div id="message" class="updated fade"><p><strong>' . __('Options saved.', 'glc') . '</strong></p></div>';
            }

            if ( !(is_dir(CACHEPATH) || ( umask(0022) && @mkdir( CACHEPATH, 0755, true ) ) ) ) {
                echo '<div id="message" class="updated fade"><p><strong>' . __('cache directory doesn\'t exists or isn\'t writable.', 'glc') . '</strong></p></div>';
            }

            ?>
<div class="wrap">
    <h2>GravatarLocalCache</h2>

    <h3><?php _e('Cache Settings', 'glc');  ?></h3>

    <form action="<?php echo admin_url( 'options-general.php?page=' . plugin_basename( __FILE__ ) ); ?>" method="post">

        <table class="form-table">
            <tbody>

                <?php // cache time ?>
                <tr valign="top">
                    <th scope="row">
                        <label><?php echo _e('caching time', 'glc'); ?></label>
                    </th>
                    <td>
                        <input type="text" value="<?php echo self::$options['cache_time'] ?>" name="cache_time" id="cache_time" size="5" maxlength="35" />
                        <br />
                        <?php _e('maximum time (in hours) the gravatar images will be cached.', 'glc'); ?>
                    </td>
                </tr>

            </tbody>
        </table>

        <input type="hidden" name="glc_submit" id="glc_submit" value="1" />
        <p class="submit">
            <input type="submit" name="Submit" value="<?php _e('Update Options »', 'glc'); ?>" />
        </p>
    </form>

</div>
        <?php
        }
        /* showAdminMenu */

    }
}

if (class_exists("GravatarLocalCache")) {
    $gravatarLocalCache = new GravatarLocalCache();
}